@@ -17,7 +17,6 @@
 #include "ui.h"
 #include "scanner.h"
 #include "book.h"
-#include "book-view.h"
 
 
 static const char *default_device = NULL;
@@ -28,18 +27,12 @@ static Scanner *scanner;
 
 static Book *book;
 
-static BookView *book_view;
-
 static gboolean scanning = FALSE;
 
 static gboolean clear_pages = FALSE;
 
 static gboolean first_autodetect = TRUE;
 
-static Orientation default_orientation = TOP_TO_BOTTOM;
-
-static int page_count = 0;
-
 
 static void
 scanner_ready_cb (Scanner *scanner)
@@ -53,7 +46,6 @@ static void
 update_scan_devices_cb (Scanner *scanner, GList *devices)
 {
     GList *dev_iter;
-    gboolean first;
 
     if (first_autodetect) {
         first_autodetect = FALSE;
@@ -87,21 +79,44 @@ static void
 scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 {
     Page *page;
-    gint height;
+    Orientation orientation = TOP_TO_BOTTOM;
+    gboolean do_crop = FALSE;
+    gchar *named_crop = NULL;
+    gint cx, cy, cw, ch;
 
     g_debug ("Page is %d pixels wide, %d pixels high, %d bits per pixel",
              info->width, info->height, info->depth);
     
+    page = book_get_page (book, -1);
+    if (page) {
+        orientation = page_get_orientation (page);
+        
+        do_crop = page_has_crop (page);
+        if (do_crop) {
+            named_crop = page_get_named_crop (page);
+            page_get_crop (page, &cx, &cy, &cw, &ch);
+        }
+    }
+
     if (clear_pages) {
-        page_count = 0;
         book_clear (book);
         clear_pages = FALSE;
     }
 
-    page = book_append_page (book, info->width, info->height, info->dpi, default_orientation);
-    page_start (page);
+    page = book_append_page (book, info->width, info->height, info->dpi, orientation);
+    if (do_crop) {
+        if (named_crop)  {
+            page_set_named_crop (page, named_crop);
+            g_free (named_crop);
+        }
+        else
+            page_set_custom_crop (page, cw, ch);
+        page_move_crop (page, cx, cy);
+    }
+    
+    ui_set_selected_page (ui, page);
 
-    page_count++;
+    page_start (page);
 }
 
 
@@ -110,7 +125,7 @@ scanner_line_cb (Scanner *scanner, ScanLine *line)
 {
     Page *page;
 
-    page = book_get_page (book, page_count - 1);
+    page = book_get_page (book, book_get_n_pages (book) - 1);
     page_parse_scan_line (page, line);
 }
 
@@ -120,7 +135,7 @@ scanner_image_done_cb (Scanner *scanner)
 {
     Page *page;
     
-    page = book_get_page (book, page_count - 1);
+    page = book_get_page (book, book_get_n_pages (book) - 1);
     page_finish (page);
     ui_set_have_scan (ui, TRUE);
 }
@@ -137,54 +152,45 @@ scanner_failed_cb (Scanner *scanner, GError *error)
 
 
 static void
-redraw_cb (BookView *view)
+scan_cb (SimpleScan *ui, const gchar *device, const gchar *profile_name, gboolean continuous, gboolean replace)
 {
-    ui_redraw_preview (ui);  
-}
+    struct {
+        const gchar *name;
+        gint dpi;
+        ScanMode mode;
+        const gchar *file_name;
+    } profiles[] = 
+    {
+        { "text", 200, SCAN_MODE_LINEART,
+          /* Default name for PDF documents */
+          _("Scanned Document.pdf") },
+        { "photo", 400, SCAN_MODE_COLOR,
+          /* Default name for JPEG documents */
+          _("Scanned Document.jpeg") },
+        { "raw", 800, SCAN_MODE_COLOR,
+          /* Default name for PNG documents */
+          _("Scanned Document.png") },
+        { NULL, 75, SCAN_MODE_COLOR,
+          /* Default name for JPEG documents */
+          _("Scanned Document.jpeg") }                
+    };
+    gint i;
 
-
-static void
-scan_cb (SimpleScan *ui, const gchar *device, const gchar *document_type, gboolean continuous, gboolean replace)
-{
-    gint dpi;
-
-    g_debug ("Requesting scan of type %s from device '%s'", document_type, device);
+    g_debug ("Requesting scan of type %s from device '%s'", profile_name, device);
 
     scanning = TRUE;
     ui_set_have_scan (ui, FALSE);
     ui_set_scanning (ui, TRUE);
     
-    if (strcmp (document_type, "photo") == 0) {
-        ui_set_default_file_name (ui,
-                                  /* Default name for JPEG documents */
-                                  _("Scanned Document.jpeg"));
-        dpi = 400;
-    }
-    else if (strcmp (document_type, "text") == 0) {
-        ui_set_default_file_name (ui,
-                                  /* Default name for PDF documents */
-                                  _("Scanned Document.pdf"));
-        dpi = 200;
-    }
-    else if (strcmp (document_type, "raw") == 0) {
-        ui_set_default_file_name (ui,
-                                  /* Default name for PNG documents */
-                                  _("Scanned Document.png"));
-        dpi = 800;
-    }
-    /* Draft or unknown */
-    else
-    {
-        ui_set_default_file_name (ui, _("Scanned Document.jpeg"));
-        dpi = 75;
-    }
+    /* Find this profile */
+    for (i = 0; profiles[i].name && strcmp (profiles[i].name, profile_name) != 0; i++);
 
     if (replace)
         clear_pages = TRUE;
-
-    scanner_scan (scanner, device, NULL, dpi, "Color", 8, continuous);
-    //scanner_scan (scanner, device, "Flatbed", 50, "Color", 8, continuous);
-    //scanner_scan (scanner, device, "Automatic Document Feeder", 200, "Color", 8, continuous);
+    ui_set_default_file_name (ui, profiles[i].file_name);
+    scanner_scan (scanner, device, NULL, profiles[i].dpi, profiles[i].mode, 8, continuous);
+    //scanner_scan (scanner, device, "Flatbed", 50, mode, 8, continuous);
+    //scanner_scan (scanner, device, "Automatic Document Feeder", 200, mode, 8, continuous);
 }
 
 
@@ -196,22 +202,25 @@ cancel_cb (SimpleScan *ui)
 
 
 static void
-rotate_left_cb (SimpleScan *ui)
+add_default_page ()
 {
-    Page *page;
-    page = book_view_get_selected (book_view);
-    page_rotate_left (page);
-    default_orientation = page_get_orientation (page);
+    if (book_get_n_pages (book) > 0)
+        return;
+
+    /* Start with A4 white image at 72dpi */
+    /* TODO: Should be like the last scanned image for the selected scanner */
+    book_append_page (book, 595, 842, 72, TOP_TO_BOTTOM);
+
+    /* Remove this page on the next scan */
+    clear_pages = TRUE;
 }
 
 
 static void
-rotate_right_cb (SimpleScan *ui)
+page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 {
-    Page *page;
-    page = book_view_get_selected (book_view);
-    page_rotate_right (page);
-    default_orientation = page_get_orientation (page);
+    /* Ensure always one page */
+    add_default_page ();
 }
 
 
@@ -260,9 +269,14 @@ save_cb (SimpleScan *ui, gchar *uri)
 
 
 static void
-print_cb (SimpleScan *ui, cairo_t *context)
+email_cb (SimpleScan *ui)
 {
-    book_print (book, context);
+    GError *error = NULL;
+    g_spawn_command_line_async ("xdg-email", &error);
+    if (error) {
+        g_warning ("Unable to start email: %s", error->message);
+        g_error_free (error);
+    }
 }
 
 
@@ -361,26 +375,17 @@ main(int argc, char **argv)
     get_options (argc, argv);
 
     book = book_new ();
-    /* Start with A4 white image at 72dpi */
-    /* TODO: Should be like the last scanned image for the selected scanner */
-    book_append_page (book, 595, 842, 72, default_orientation);
-    page_count++;
+    g_signal_connect (book, "page-removed", G_CALLBACK (page_removed_cb), NULL);
+    add_default_page ();
 
     ui = ui_new ();
+    ui_set_book (ui, book);
     g_signal_connect (ui, "start-scan", G_CALLBACK (scan_cb), NULL);
     g_signal_connect (ui, "stop-scan", G_CALLBACK (cancel_cb), NULL);
-    g_signal_connect (ui, "rotate-left", G_CALLBACK (rotate_left_cb), NULL);
-    g_signal_connect (ui, "rotate-right", G_CALLBACK (rotate_right_cb), NULL);
     g_signal_connect (ui, "save", G_CALLBACK (save_cb), NULL);
-    g_signal_connect (ui, "print", G_CALLBACK (print_cb), NULL);
+    g_signal_connect (ui, "email", G_CALLBACK (email_cb), NULL);
     g_signal_connect (ui, "quit", G_CALLBACK (quit_cb), NULL);
 
-    book_view = book_view_new ();
-    book_view_set_widget (book_view, ui_get_preview_widget (ui)); // FIXME
-    book_view_set_book (book_view, book);
-    
-    ui_set_zoom_adjustment (ui, book_view_get_zoom_adjustment (book_view));
-    
     scanner = scanner_new ();
     g_signal_connect (G_OBJECT (scanner), "ready", G_CALLBACK (scanner_ready_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "update-devices", G_CALLBACK (update_scan_devices_cb), NULL);
